# -*- coding: utf-8 -*-
"""
Trac WebAdmin plugin for administration of custom fields.

License: BSD

(c) 2005-2012 ::: www.CodeResort.com - BV Network AS (simon-code@bvnetwork.no)
(c) 2007-2009 ::: www.Optaros.com (.....)
"""
from pkg_resources import get_distribution, parse_version, resource_filename

from trac.admin.api import IAdminPanelProvider
from trac.config import Option
from trac.core import *
from trac.web.chrome import (
    Chrome, ITemplateProvider, add_script, add_script_data, add_warning)

from customfieldadmin.api import CustomFields, _


class CustomFieldAdminPage(Component):

    implements(ITemplateProvider, IAdminPanelProvider)

    def __init__(self):
        CustomFields(self.env)  # load our message catalog

    # IAdminPanelProvider methods

    def get_admin_panels(self, req):
        if 'TICKET_ADMIN' in req.perm('admin', 'ticket/customfields'):
            yield ('ticket', _("Ticket System"),
                   'customfields', _("Custom Fields"))

    def render_admin_panel(self, req, cat, page, customfield):
        req.perm('admin', 'ticket/customfields').require('TICKET_ADMIN')

        cf_api = CustomFields(self.env)
        add_script_data(req, field_formats=cf_api.field_formats)
        add_script(req, 'customfieldadmin/js/customfieldadmin.js')

        def _customfield_from_req(self, req):
            cfield = {
                'name': req.args.get('name',''),
                'label': req.args.get('label',''),
                'type': req.args.get('type',''),
                'value': req.args.get('value',''),
                'options': [x.strip().encode('utf-8') for x in \
                            req.args.get('options','').split("\n")],
                'cols': req.args.get('cols',''),
                'rows': req.args.get('rows',''),
                'order': req.args.get('order', ''),
                'format': req.args.get('format', '')
            }
            return cfield

        cf_admin = {'field_types': cf_api.field_types}


        # Detail view?
        if customfield:
            cfield = None
            for a_cfield in cf_api.get_custom_fields():
                if a_cfield['name'] == customfield:
                    cfield = a_cfield
                    break
            if not cfield:
                raise TracError(_("Custom field %(name)s does not exist.",
                                  name=customfield))
            if req.method == 'POST':
                if req.args.get('save'):
                    cfield.update(_customfield_from_req(self, req))
                    cf_api.update_custom_field(cfield)
                    req.redirect(req.href.admin(cat, page))
                elif req.args.get('cancel'):
                    req.redirect(req.href.admin(cat, page))
            if 'options' in cfield:
                optional_line = ''
                if cfield.get('optional', False):
                    optional_line = "\n\n"
                cfield['options'] = \
                        optional_line + "\n".join(cfield['options'])
            cf_admin['cfield'] = cfield
            cf_admin['cf_display'] = 'detail'
        else:
            if req.method == 'POST':
                # Add Custom Field
                if req.args.get('add') and req.args.get('name'):
                    cfield = _customfield_from_req(self, req)
                    cf_api.update_custom_field(cfield, create=True)
                    req.redirect(req.href.admin(cat, page))

                # Remove Custom Field
                elif req.args.get('remove') and req.args.get('sel'):
                    sel = req.args.get('sel')
                    sel = isinstance(sel, list) and sel or [sel]
                    if not sel:
                        raise TracError(_("No custom field selected"))
                    for name in sel:
                        cfield =  {'name': name}
                        cf_api.delete_custom_field(cfield)
                    req.redirect(req.href.admin(cat, page))

                elif req.args.get('apply'):
                    # Change order
                    order = dict([(key[6:], req.args.get(key)) for key
                                  in req.args.keys()
                                  if key.startswith('order_')])
                    cfields = cf_api.get_custom_fields()
                    for current_cfield in cfields:
                        new_order = order.get(current_cfield['name'], 0)
                        if new_order:
                            current_cfield['order'] = new_order
                            cf_api.update_custom_field(current_cfield)
                    req.redirect(req.href.admin(cat, page))

            cfields = []
            orders_in_use = []
            for item in cf_api.get_custom_fields():
                item['href'] = req.href.admin(cat, page, item['name'])
                item['registry'] = ('ticket-custom',
                                            item['name']) in Option.registry
                cfields.append(item)
                orders_in_use.append(int(item.get('order')))
            cf_admin['cfields'] = cfields
            cf_admin['cf_display'] = 'list'
            if sorted(orders_in_use) != range(1, len(cfields)+1):
                add_warning(req, _("Custom Fields are not correctly sorted. " \
                         "This may affect appearance when viewing tickets."))

        if hasattr(Chrome(self.env), 'jenv'):
            cf_admin['domain'] = 'customfieldadmin'
            return 'customfieldadmin_jinja.html', cf_admin
        else:
            return 'customfieldadmin_genshi.html', cf_admin

    # ITemplateProvider methods

    def get_templates_dirs(self):
        return [resource_filename(__name__, 'templates')]

    def get_htdocs_dirs(self):
        return [('customfieldadmin', resource_filename(__name__, 'htdocs'))]



# -*- coding: utf-8 -*-
"""
License: BSD

(c) 2005-2012 ::: www.CodeResort.com - BV Network AS (simon-code@bvnetwork.no)
"""

import unittest

from trac.perm import PermissionSystem
from trac.test import EnvironmentStub, MockRequest
from trac.web.api import RequestDone

from customfieldadmin.admin import CustomFieldAdminPage
from customfieldadmin.api import CustomFields


class CustomFieldAdminPageTestCase(unittest.TestCase):

    def setUp(self):
        self.env = EnvironmentStub()
        ps = PermissionSystem(self.env)
        ps.grant_permission('admin', 'TICKET_ADMIN')
        self.plugin = CustomFieldAdminPage(self.env)
        self.api = CustomFields(self.env)

    def tearDown(self):
        self.env.destroy_db()

    def test_create(self):
        req = MockRequest(self.env, path_info='/admin/customfields',
                method='POST', authname='admin', args={
                    'add': True,
                     'name': "test",
                     'type': "textarea",
                     'label': "testing",
                     'format': "wiki",
                     'row': '9',
                }
        )
        with self.assertRaises(RequestDone):
            self.plugin.render_admin_panel(req, 'ticket', 'customfields', None)
        self.assertEquals([
            (u'test', u'textarea'),
            (u'test.format', u'wiki'),
            (u'test.label', u'testing'),
            (u'test.order', u'1'),
            (u'test.rows', u'5'),
            (u'test.value', u''),
        ], sorted(list(self.env.config.options('ticket-custom'))))

    def test_add_optional_select(self):
        req = MockRequest(self.env, path_info='/admin/customfields',
                method='POST', authname='admin', args={
                    'add': True,
                    'name': "test",
                    'type': "select",
                    'label': "testing",
                    'options': "\r\none\r\ntwo"
                }
        )
        with self.assertRaises(RequestDone):
            self.plugin.render_admin_panel(req, 'ticket', 'customfields', None)
        self.assertEquals([
            (u'test', u'select'), (u'test.label', u'testing'),
            (u'test.options', u'|one|two'), (u'test.order', u'1'),
            (u'test.value', u'')
        ], sorted(list(self.env.config.options('ticket-custom'))))

    def test_apply_optional_select(self):
        # Reuse the added custom field that test verified to work
        self.test_add_optional_select()
        self.assertEquals('select',
                            self.env.config.get('ticket-custom', 'test'))
        # Now check that details are maintained across order change
        # that reads fields, deletes them, and creates them again
        # http://trac-hacks.org/ticket/1834#comment:5
        req = MockRequest(self.env, path_info='/admin/customfields',
                method='POST', authname='admin', args={
                    'apply': True,
                    'order_test': '2'
            }
        )
        with self.assertRaises(RequestDone):
            self.plugin.render_admin_panel(req, 'ticket', 'customfields', None)
        self.assertEquals([
            (u'test', u'select'), (u'test.label', u'testing'),
            (u'test.options', u'|one|two'), (u'test.order', u'2'),
            (u'test.value', u'')
        ], sorted(list(self.env.config.options('ticket-custom'))))

    def test_edit_optional_select(self):
        self.test_add_optional_select()
        self.assertEquals('select',
                self.env.config.get('ticket-custom', 'test'))
        req = MockRequest(self.env, path_info='/admin/customfields',
                method='POST', authname='admin', args={
                    'save': True, 'name': u'test', 'label': u'testing',
                    'type': u'select', 'value': u'',
                    'options': u'\r\none\r\ntwo'
                }
        )
        with self.assertRaises(RequestDone):
            self.plugin.render_admin_panel(req, 'ticket', 'customfields',
                                           'test')
        self.assertEquals([
            (u'test', u'select'), (u'test.label', u'testing'),
            (u'test.options', u'|one|two'), (u'test.order', u'2'),
            (u'test.value', u'')
        ], sorted(list(self.env.config.options('ticket-custom'))))

    def test_order_with_mismatched_keys(self):
        # http://trac-hacks.org/ticket/11540
        self.api.create_custom_field({
            'name': u'one', 'format': 'plain', 'value': '',
            'label': u'One', 'type': u'text', 'order': 1
        })
        req = MockRequest(self.env, path_info='/admin/customfields',
                method='POST', authname='admin', args={
                    'apply': True,
                    'order_two': '1'
                }
        )
        with self.assertRaises(RequestDone):
            self.plugin.render_admin_panel(req, 'ticket', 'customfields', None)



# -*- coding: utf-8 -*-
"""
API for administrating custom ticket fields in Trac.
Supports creating, getting, updating and deleting custom fields.

License: BSD

(c) 2005-2012 ::: www.CodeResort.com - BV Network AS (simon-code@bvnetwork.no)
"""

import pkg_resources
import re

from trac.core import *
from trac.ticket.api import TicketSystem
from trac.util.translation import domain_functions

add_domain, _, tag_ = \
    domain_functions('customfieldadmin', ('add_domain', '_', 'tag_'))


class CustomFields(Component):
    """ These methods should be part of TicketSystem API/Data Model.
    Adds update_custom_field and delete_custom_field methods.
    (The get_custom_fields is already part of the API - just redirect here,
     and add option to only get one named field back.)

    Input to methods is a 'cfield' dict supporting these keys:
        name = name of field (ascii alphanumeric only)
        type = text|checkbox|select|radio|textarea|time
        label = label description
        value = default value for field content
        options = options for select and radio types
                  (list, leave first empty for optional)
        rows = number of rows for text area
        order = specify sort order for field
        format = text|wiki|reference|list (for text)
        format = text|wiki (for textarea)
        format = relative|date|datetime (for time)
    """

    config_options = ['label', 'value', 'options', 'rows', 'order', 'format']
    field_types = ['text', 'checkbox', 'select', 'radio', 'textarea', 'time']
    field_formats = {
        'text': ['plain', 'wiki', 'reference', 'list'],
        'textarea': ['plain', 'wiki'],
        'time': ['relative', 'date', 'datetime'],
    }

    def __init__(self):
        # bind the 'customfieldadmin' catalog to the specified locale directory
        try:
            locale_dir = pkg_resources.resource_filename(__name__, 'locale')
        except KeyError:
            # No 'locale', hence no compiled message catalogs. Ignore.
            pass
        else:
            add_domain(self.env.path, locale_dir)

    def get_custom_fields(self, cfield=None):
        """ Returns the custom fields from TicketSystem component.
        Use a cfdict with 'name' key set to find a specific custom field only.
        """
        items = TicketSystem(self.env).get_custom_fields()
        for item in items:
            if item['type'] == 'textarea':
                item['rows'] = item.pop('height')
            if cfield and item['name'] == cfield['name']:
                return item  # only return specific item with cfname
        if cfield:
            return None   # item not found
        else:
            return items  # return full list

    def verify_custom_field(self, cfield, create=True):
        """ Basic validation of the input for modifying or creating
        custom fields. """
        # Requires 'name' and 'type'
        if not (cfield.get('name') and cfield.get('type')):
            raise TracError(
                    _("Custom field requires attributes 'name' and 'type'."))
        # Use lowercase custom fieldnames only
        cfield['name'] = cfield['name'].lower()
        # Check field name is not reserved
        tktsys = TicketSystem(self.env)
        if cfield['name'] in tktsys.reserved_field_names:
            raise TracError(_("Field name '%(name)s' is reserved",
                              name=cfield['name']))
        # Only alphanumeric characters (and [-_]) allowed for custom fieldname
        if re.search('^[a-z][a-z0-9_]+$', cfield['name']) == None:
            raise TracError(_("Only alphanumeric characters allowed for " \
                             "custom field name ('a-z' or '0-9' or '_'), " \
                             "with 'a-z' as first character."))
        # Name must begin with a character - anything else not supported by Trac
        if not cfield['name'][0].isalpha():
            raise TracError(
                    _("Custom field name must begin with a character (a-z)."))
        # Check that it is a valid field type
        if not cfield['type'] in self.field_types:
            raise TracError(_("%(field_type)s is not a valid field type",
                              field_type=cfield['type']))
        # Check that field does not already exist
        # (if modify it should already be deleted)
        if create and self.config.get('ticket-custom', cfield['name']):
            raise TracError(_("Can not create as field already exists."))
        if create and [f for f in tktsys.fields
                         if f['name'] == cfield['name']]:
            raise TracError(_("Can't create a custom field with the "
                              "same name as a built-in field."))

    def create_custom_field(self, cfield):
        """ Create the new custom fields (that may just have been deleted as
        part of 'modify'). In `cfield`, 'name' and 'type' keys are required.
        Note: Caller is responsible for verifying input before create."""
        # Need count pre-create for correct order
        count_current_fields = len(self.get_custom_fields())
        # Set the mandatory items
        self.config.set('ticket-custom', cfield['name'], cfield['type'])
        # Label = capitalize fieldname if not present
        self.config.set('ticket-custom', cfield['name'] + '.label',
                        cfield.get('label') or cfield['name'].capitalize())
        # Optional items
        if 'value' in cfield:
            self.config.set('ticket-custom', cfield['name'] + '.value',
                                                        cfield['value'])
        if cfield['type'] in ('select', 'radio') and \
                'options' in cfield:
            if cfield.get('optional', False) and '' not in cfield['options']:
                self.config.set('ticket-custom', cfield['name'] + '.options',
                                '|' + '|'.join(cfield['options']))
            else:
                self.config.set('ticket-custom', cfield['name'] + '.options',
                               '|'.join(cfield['options']))
        if 'format' in cfield and \
                cfield['type'] in ('text', 'textarea', 'time') and \
                cfield['format'] in self.field_formats[cfield['type']]:
            self.config.set('ticket-custom', cfield['name'] + '.format',
                            cfield['format'])
        # Textarea
        if cfield['type'] == 'textarea':
            rows = cfield.get('rows', 0) and int(cfield.get('rows', 0)) > 0 \
                                                and cfield.get('rows') or 5
            self.config.set('ticket-custom', cfield['name'] + '.rows', rows)
        # Order
        order = cfield.get('order') or count_current_fields + 1
        self.config.set('ticket-custom', cfield['name'] + '.order', order)
        self._save(cfield)

    def update_custom_field(self, cfield, create=False):
        """ Updates a custom field. Option to 'create' is kept in order to keep
        the API backwards compatible. """
        if create:
            self.verify_custom_field(cfield)
            self.create_custom_field(cfield)
            return
        # Check input, then delete and save new
        if not self.get_custom_fields(cfield=cfield):
            raise TracError(_("Custom Field '%(name)s' does not exist. " \
                    "Cannot update.", name=cfield.get('name') or '(none)'))
        self.verify_custom_field(cfield, create=False)
        self.delete_custom_field(cfield, modify=True)
        self.create_custom_field(cfield)

    def delete_custom_field(self, cfield, modify=False):
        """ Deletes a custom field. Input is a dictionary
        (see update_custom_field), but only ['name'] is required.
        """
        if not self.config.get('ticket-custom', cfield['name']):
            return # Nothing to do here - cannot find field
        if not modify:
            # Permanent delete - reorder later fields to lower order
            order_to_delete = self.config.getint('ticket-custom',
                                                 cfield['name']+'.order')
            cfs = self.get_custom_fields()
            for field in cfs:
                if field['order'] > order_to_delete:
                    self.config.set('ticket-custom', field['name'] + '.order',
                                    field['order'] - 1 )
        supported_options = [cfield['name']] + \
            [cfield['name'] + '.' + opt for opt in self.config_options]
        for option, _value in self.config.options('ticket-custom'):
            if modify and not option in supported_options:
                # Only delete supported options when modifying
                # http://trac-hacks.org/ticket/8188
                continue
            if option == cfield['name'] \
                    or option.startswith(cfield['name'] + '.'):
                self.config.remove('ticket-custom', option)
        # Persist permanent deletes
        if not modify:
            self._save(cfield)

    def _save(self, cfield=None):
        """ Saves a value, clear caches if needed / supported. """
        self.config.save()
        del TicketSystem(self.env).custom_fields
        # Re-populate contents of cfield with new values and defaults
        if cfield:
            stored = self.get_custom_fields(cfield=cfield)
            if stored: # created or updated (None for deleted so just ignore)
                cfield.update(stored)



# -*- coding: utf-8 -*-
"""
License: BSD

(c) 2005-2012 ::: www.CodeResort.com - BV Network AS (simon-code@bvnetwork.no)
"""

import unittest

from trac.core import TracError
from trac.test import EnvironmentStub, Mock
from trac.util.html import html as tag

from customfieldadmin.api import CustomFields, tag_


class CustomFieldApiTestCase(unittest.TestCase):

    def setUp(self):
        self.env = EnvironmentStub()
        self.cf_api = CustomFields(self.env)

    def tearDown(self):
        self.env.destroy_db()

    def test_create(self):
        for f in ['one', 'two', 'three']:
            cfield = {'name': f, 'type': 'text'}
            self.cf_api.create_custom_field(cfield)
        self.assertEquals([{
                'custom': True, 'name': u'one', 'format': 'plain',
                'value': '', 'label': u'One', 'type': u'text', 'order': 1
            }, {
                'custom': True, 'name': u'two', 'format': 'plain',
                'value': '', 'label': u'Two', 'type': u'text', 'order': 2
            }, {
                'custom': True, 'name': u'three', 'format': 'plain',
                'value': '', 'label': u'Three', 'type': u'text', 'order': 3
            }
        ], self.cf_api.get_custom_fields())

    def test_update(self):
        cfield = {'name': 'foo', 'type': 'text'}
        self.cf_api.create_custom_field(cfield)
        self.assertEquals({
            'custom': True, 'name': u'foo', 'format': 'plain',
            'value': '', 'label': u'Foo', 'type': u'text', 'order': 1
        }, cfield)

        cfield['label'] = 'Answer'
        cfield['value'] = '42'
        self.cf_api.update_custom_field(cfield=cfield)
        self.assertEquals({
            'custom': True, 'name': u'foo', 'format': 'plain',
            'value': '42', 'label': u'Answer', 'type': u'text', 'order': 1
        }, cfield)

    def test_update_textarea(self):
        cfield = {'name': 'foo', 'type': 'textarea'}
        self.cf_api.create_custom_field(cfield)

        self.assertEquals({
            'custom': True, 'name': u'foo', 'format': 'plain', 'value': '',
            'label': u'Foo', 'type': u'textarea', 'order': 1, 'rows': 5
        }, cfield)

        cfield['rows'] = 3
        self.cf_api.update_custom_field(cfield=cfield)

        self.assertEquals({
            'custom': True, 'name': u'foo', 'format': 'plain', 'value': '',
            'label': u'Foo', 'type': u'textarea', 'order': 1, 'rows': 3
        }, cfield)

    def test_update_non_existing(self):
        with self.assertRaises(Exception) as cm:
            self.cf_api.update_custom_field(cfield={'name': 'no_field'})
        self.assertTrue("'no_field'" in cm.exception.message)
        self.assertTrue('does not exist' in cm.exception.message)

    def test_update_non_existing_no_name(self):
        with self.assertRaises(Exception) as cm:
            self.cf_api.update_custom_field(cfield={})
        self.assertTrue("'(none)'" in cm.exception.message)
        self.assertTrue('does not exist' in cm.exception.message)

    def test_delete(self):
        for f in ['one', 'two', 'three']:
            cfield = {'name': f, 'type': 'text'}
            self.cf_api.create_custom_field(cfield)
        self.assertIn(('two', 'text'),
                self.env.config.options('ticket-custom'))
        self.cf_api.delete_custom_field({'name': 'two'})
        self.assertNotIn(('two', 'text'),
                self.env.config.options('ticket-custom'))
        # Note: Should also reorder higher-ordered items
        self.assertEquals([{
                'custom': True, 'name': u'one', 'format': 'plain',
                'value': '', 'label': u'One', 'type': u'text', 'order': 1
            }, {
                'custom': True, 'name': u'three', 'format': 'plain',
                'value': '', 'label': u'Three', 'type': u'text', 'order': 2
            }
        ], self.cf_api.get_custom_fields())
        self.assertIsNone(
                self.cf_api.get_custom_fields(cfield={'name': 'two'}))

    def test_delete_unknown_options(self):
        cf = {'name': 'foo', 'type': 'text', 'label': 'Foo'}
        self.cf_api.create_custom_field(cf)
        self.assertEquals('text',
                self.env.config.get('ticket-custom', 'foo'))
        self.assertEquals('Foo',
                self.env.config.get('ticket-custom', 'foo.label'))
        self.env.config.set('ticket-custom', 'foo.answer', '42')
        self.cf_api.delete_custom_field(cf, modify=False)
        self.assertEquals('',
                self.env.config.get('ticket-custom', 'foo'))
        self.assertEquals('',
                self.env.config.get('ticket-custom', 'foo.label'))
        self.assertEquals('',
                self.env.config.get('ticket-custom', 'foo.answer'))

    def test_not_delete_unknown_options_for_modify(self):
        cf = {'name': 'foo', 'type': 'text', 'label': 'Foo'}
        self.cf_api.create_custom_field(cf)
        self.assertEquals('text',
                self.env.config.get('ticket-custom', 'foo'))
        self.assertEquals('Foo',
                self.env.config.get('ticket-custom', 'foo.label'))
        self.env.config.set('ticket-custom', 'foo.answer', '42')
        self.cf_api.delete_custom_field(cf, modify=True)
        self.assertEquals('',
                self.env.config.get('ticket-custom', 'foo'))
        self.assertEquals('',
                self.env.config.get('ticket-custom', 'foo.label'))
        self.assertEquals('42',
                self.env.config.get('ticket-custom', 'foo.answer'))

    def test_verify_unknown_type(self):
        self.env.config.set('ticket-custom', 'one', 'foo_type')
        fields = self.cf_api.get_custom_fields()
        self.assertEquals(1, len(fields))
        with self.assertRaises(TracError) as cm:
            self.cf_api.verify_custom_field(fields[0], create=False)
        self.assertTrue('foo_type' in cm.exception.message)

    def test_verify_reserved_names(self):
        cf = {'name': 'group', 'type': 'text', 'label': 'Group'}
        with self.assertRaises(TracError):
            self.cf_api.verify_custom_field(cf)
        cf = {'name': 'Group', 'type': 'text'}
        with self.assertRaises(TracError):
            self.cf_api.verify_custom_field(cf)
        cf = {'name': 'group_', 'type': 'text', 'label': 'Group'}
        self.cf_api.verify_custom_field(cf)  # no errors


class CustomFieldL10NTestCase(unittest.TestCase):

    def setUp(self):
        self.env = EnvironmentStub()
        self.cf_api = CustomFields(self.env)

    def tearDown(self):
        self.env.destroy_db()

    def test_translation_function(self):
        from customfieldadmin.api import _
        self.assertEquals('foo bar', _("foo bar"))
        self.assertEquals('foo bar', _("foo %(bar)s", bar='bar'))

    def test_translation_function_tag(self):
        self.assertEquals('<p>foo bar</p>', str(tag_(tag.p('foo bar'))))
        self.assertEquals('<p>foo bar</p>',
                    unicode(tag_(tag.p('foo %(bar)s' % {'bar': 'bar'}))))



# -*- coding: utf-8 -*-
"""
License: BSD

(c) 2005-2011 ::: www.CodeResort.com - BV Network AS (simon-code@bvnetwork.no)
"""

from unittest import TestSuite, makeSuite

def test_suite():
    suite = TestSuite()
    from customfieldadmin.tests import api
    suite.addTest(makeSuite(api.CustomFieldApiTestCase))
    suite.addTest(makeSuite(api.CustomFieldL10NTestCase))
    from customfieldadmin.tests import admin
    suite.addTest(makeSuite(admin.CustomFieldAdminPageTestCase))
    return suite



#!/usr/bin/env python

from setuptools import setup

from trac.dist import get_l10n_cmdclass


extra = {}
cmdclass = get_l10n_cmdclass()
if cmdclass:
    extra['cmdclass'] = cmdclass


setup(name='TracCustomFieldAdmin',
      version='0.4.0',
      packages=['customfieldadmin'],
      author='CodeResort.com & Optaros.com',
      description='Admin panel for managing Trac ticket custom fields.',
      url='https://trac-hacks.org/wiki/CustomFieldAdminPlugin',
      license='BSD',
      classifiers=[
          'Framework :: Trac',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: BSD License',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
      ],
      entry_points={
          'trac.plugins': [
              'customfieldadmin.api = customfieldadmin.api',
              'customfieldadmin.admin = customfieldadmin.admin',
          ]
      },
      exclude_package_data={'': ['tests/*']},
      test_suite = 'customfieldadmin.tests.test_suite',
      tests_require = [],
      package_data={
          'customfieldadmin' : [
              'htdocs/css/*.css',
              'htdocs/js/*.js',
              'templates/*.html',
              'locale/*/LC_MESSAGES/*.mo'
          ]
      },
      extras_require = {'Babel': 'Babel>=0.9.6'},
      **extra
 )
